@@ -1,8 +1,13 @@
 class TrainerLoopConfig():
     def __init__(self):
-        self.train = None
+        self._train = None
 
-    def process_batch(self, x_batch, y_batch):
+    @property
+    def training(self) -> bool:
+        return self._train
+
+    def process_batch(self, batch):
+        *x_batch, y_batch = batch
         return x_batch, y_batch
 
     def forward_call(self, model, x_batch):
