@@ -3,7 +3,7 @@ import time
 import pandas as pd
 from enum import Enum
 from collections import OrderedDict
-from .utils import Progbar
+from .utils._keras_progress_bar import Progbar, _log_format
 
 from copy import deepcopy
 
@@ -14,11 +14,12 @@ class Events(Enum):
     ON_EPOCH_BEGIN = 'on_epoch_begin'
     ON_TRAIN_BEGIN = 'on_train_begin'
     ON_TRAIN_END = 'on_train_end'
+    ON_BATCH_BEGIN = 'on_batch_begin'
+    ON_BATCH_END = 'on_batch_end'
 
 
 class StopTrainingError(Exception):
-    def __init__(self):
-        pass
+    pass
 
 @torch.no_grad()
 def calc_metrics(y_pred, y_true, metrics_dic: OrderedDict):
@@ -74,8 +75,34 @@ class MetricsRecorder():
         metrics.update(self.average_epoch_metrics(free_memory))
         return metrics
 
+class autocast():
+    """
+    https://github.com/pytorch/pytorch/blob/master/torch/cuda/amp/autocast_mode.py
+    """
+    def __init__(self, enabled, device) -> None:
+        super().__init__()
+        self._enabled = enabled
 
-class Trainer(object):
+        if enabled:
+            assert device == 'cuda', f"`use_amp=True` can be used for CUDA device only. While we got `device={device}`."
+
+    def __enter__(self):
+        if not self._enabled:
+            return
+        self.prev = torch.is_autocast_enabled()
+        torch.set_autocast_enabled(self._enabled)
+        torch.autocast_increment_nesting()
+
+    def __exit__(self, *args):
+        if not self._enabled:
+            return
+        # Drop the cache when we exit to a nesting level that's outside any instance of autocast.
+        if torch.autocast_decrement_nesting() == 0:
+            torch.clear_autocast_cache()
+        torch.set_autocast_enabled(self.prev)
+        return False
+
+class Trainer():
     def __init__(self, **kwargs) -> None:
         for key, value in kwargs.items():
             setattr(self, key, value)
@@ -84,7 +111,10 @@ class Trainer(object):
             self.loop_config = _default_trainer_loop_config
     
     def register_callbacks(self, callbacks):
-        self.event_dict = {Events(k).value: list() for k in Events}
+        self.event_dict = {Events(k).value: [] for k in Events}
+
+        if callbacks is None:
+            return
 
         if not isinstance(callbacks, list):
             callbacks = [callbacks]
@@ -99,14 +129,14 @@ class Trainer(object):
 
     def get_loop_config(self, train):
         loop = deepcopy(self.loop_config)
-        loop.train = train
+        loop._train = train
         return loop
 
     def run(self, train_loader, val_loader, max_epochs, verbose, use_amp, accum_grad_steps):
         self.data_loaders = train_loader, val_loader
         self.max_epochs = max_epochs
         self.logger = Logger(self, verbose=verbose)
-        self.use_amp = use_amp if self.device == 'cuda' else False
+        self.use_amp = use_amp
         self.accum_grad_steps = accum_grad_steps
         
         self.logger.on_train_begin(train_loader, val_loader)
@@ -134,22 +164,32 @@ class Trainer(object):
     def train_on_epoch(self, data_loader):
         self.model.train()
         metrics_rec = MetricsRecorder(self.metrics, self.epoch_metrics)
-        grad_scaler = torch.cuda.amp.GradScaler(enabled=self.use_amp)
         loop = self.get_loop_config(train=True)
 
+        if self.use_amp:
+            grad_scaler = torch.cuda.amp.GradScaler()
+            def optimizer_step():
+                grad_scaler.step(self.optimizer)
+                grad_scaler.update()
+            loss_backward = lambda loss: grad_scaler.scale(loss).backward()
+        else:
+            optimizer_step = lambda: self.optimizer.step()
+            loss_backward = lambda loss: loss.backward()
+
         self.optimizer.zero_grad()
 
         batch_idx, max_batch_idx = 0, len(data_loader)
         for batch in data_loader:
-            *x_batch, y_batch = [i.to(device=self.device) for i in batch]
-            x_batch, y_batch = loop.process_batch(x_batch, y_batch)
+            self.__fire_event(Events.ON_BATCH_BEGIN)
+            batch = [i.to(device=self.device) for i in batch]
+            x_batch, y_batch = loop.process_batch(batch)
             
-            with torch.cuda.amp.autocast(enabled=self.use_amp):
+            with autocast(self.use_amp, self.device):
                 y_batch_pred = loop.forward_call(self.model, x_batch)
                 loss = self.loss(y_batch_pred, y_batch)
                 loss = loss / self.accum_grad_steps
 
-            grad_scaler.scale(loss).backward()
+            loss_backward(loss)
 
             # https://discuss.pytorch.org/t/pytorch-gradient-accumulation/55955
 
@@ -158,8 +198,7 @@ class Trainer(object):
 
             if _is_accum_end or _is_last_batch:
                 loop.prepare_for_optimizer_step(self.model)
-                grad_scaler.step(self.optimizer)
-                grad_scaler.update()
+                optimizer_step()
                 self.optimizer.zero_grad()
 
             y_batch_pred = y_batch_pred.detach().float()
@@ -169,6 +208,7 @@ class Trainer(object):
 
             self.logger.on_batch_end(metrics_rec)
             batch_idx += 1
+            self.__fire_event(Events.ON_BATCH_END)
 
         return metrics_rec.average()
 
@@ -182,10 +222,10 @@ class Trainer(object):
         loop = self.get_loop_config(train=False)
  
         for batch in data_loader:
-            *x_batch, y_batch = [i.to(device=self.device) for i in batch]
-            x_batch, y_batch = loop.process_batch(x_batch, y_batch)
+            batch = [i.to(device=self.device) for i in batch]
+            x_batch, y_batch = loop.process_batch(batch)
 
-            with torch.cuda.amp.autocast(enabled=use_amp):
+            with autocast(use_amp, self.device):
                 y_batch_pred = loop.forward_call(self.model, x_batch)
 
             y_batch_pred = y_batch_pred.detach().float()
@@ -196,7 +236,7 @@ class Trainer(object):
         return metrics_rec.average()
 
 
-class Logger(object):
+class Logger():
     def __init__(self, trainer, verbose):
         self.trainer = trainer
         self.verbose = verbose
@@ -204,7 +244,7 @@ class Logger(object):
     def on_train_begin(self, train_loader, val_loader):
         if self.verbose == 0:
             return None
-        if val_loader != None:
+        if val_loader is not None:
             print(f'Train on {len(train_loader.dataset)} samples, validate on {len(val_loader.dataset)} samples:')
         else:
             print(f'Train on {len(train_loader.dataset)} samples:')
@@ -234,10 +274,7 @@ class Logger(object):
         self.metrics.update({('val_' + k): v for k, v in val_metrics.items()})
  
         for k, v in self.metrics.items():
-            if v > 1e-3:
-                content.append('{}: {:.4f}'.format(k, v))
-            else:
-                content.append('{}: {:.4e}'.format(k, v))
+            content.append(_log_format(k, v))
 
         self.metrics['lr'] = self.trainer.optimizer.param_groups[0]['lr']
         content.append('lr: {:.1e}'.format(self.metrics['lr']).replace('.0e', 'e'))
