@@ -4,6 +4,8 @@ import torch.nn as nn
 from copy import deepcopy
 import torch.nn.functional as F
 
+from .layers._keras_layers import Lambda
+
 
 class Mish(nn.Module):
     """
@@ -15,8 +17,6 @@ class Mish(nn.Module):
 
     + https://github.com/digantamisra98/Mish
     """
-    def __init__(self):
-        super(Mish, self).__init__()
     
     def forward(self, x):
         return x * torch.tanh(F.softplus(x))
@@ -51,6 +51,9 @@ _activations_dict = OrderedDict({
     'sigmoid': nn.Sigmoid(),
     'mish': Mish(),
     'swish': Swish(),
+    'gelu': nn.GELU(),
+    'argmax': Lambda(lambda x: x.argmax(-1)),
+    'sigmoid_round': Lambda(lambda x: torch.round(torch.sigmoid(x)))
 })
 
 
@@ -60,7 +63,6 @@ def _create_activation(i):
         if name not in _activations_dict:
             raise KeyError(f'Invalid name, we support {list(_activations_dict.keys())}.')
         return deepcopy(_activations_dict[name])
-    else:
-        return i
+    return i
 
 __all__ = ['Mish', 'Swish']
\ No newline at end of file
