@@ -6,7 +6,7 @@ from collections import OrderedDict
 from torch.utils.data.dataset import Dataset
 from .._summary import summary
 
-from .._training import Trainer
+from .._training import Trainer, autocast
 from ..layers import KerasLayer
 from ..metrics import _to_metrics_dic
 from ..losses import _create_loss
@@ -198,6 +198,7 @@ class Model(torch.nn.Module):
             x, y = to_tensor(x, y)
             train_set = TensorDataset(*x, y)
         
+        val_set = None
         del x, y    # for preventing bugs
 
         if validation_data is not None:
@@ -259,13 +260,14 @@ class Model(torch.nn.Module):
     @torch.no_grad()
     def predict_dl(self, data_loader, device=None, output_numpy=True, activation=None, use_amp=False):
         self._check_keras_layer()
+
         if device is None:
             if self.compiled:
                 device = self.trainer.device
             else:
                 device = 'cuda' if torch.cuda.is_available() else 'cpu'
         self.eval().to(device=device)
-
+        
         activation = _create_activation(activation)
 
         outputs = []
@@ -276,7 +278,7 @@ class Model(torch.nn.Module):
             for i in range(len(batch)):
                 batch[i] = batch[i].to(device=device)
 
-            with torch.cuda.amp.autocast(enabled=use_amp):
+            with autocast(use_amp, device):
                 o = self(*batch)
                 outputs.append(o)
 
